@@ -1,4 +1,5 @@
 import os
+import re
 import argparse
 import _pickle as pickle
 import sqlite3
@@ -40,8 +41,8 @@ class Matcher(Preprocessor):
                best_match=False,
                output_format='csv',
                output_name=''):
-    
-    # passing a Python list is possible, call results generic name 
+
+    # passing a Python list is possible, call results generic name
     # if none is specified
     if type(query) == list:
       self.query = query
@@ -59,6 +60,21 @@ class Matcher(Preprocessor):
         # make output name using query name to proteome name
         self.output_name = f"{query.replace('.fasta', '')}_to_{proteome.split('/')[-1].split('.')[0]}"
 
+    # check if there are any discontinuous epitopes in the query
+    self.discontinuous_epitopes = []
+    for peptide in self.query:
+      try:
+        discontinuous_epitope = [(x[0], int(x[1:])) for x in peptide.split(', ')]
+        self.discontinuous_epitopes.append(discontinuous_epitope)
+      except ValueError:
+        continue
+
+    # remove discontinuous epitopes from query
+    self.query = list(set(self.query) - set([', '.join([x[0] + str(x[1]) for x in self.discontinuous_epitopes[i]]) for i in range(len(self.discontinuous_epitopes))]))
+
+    # make sure all query peptides are uppercase
+    self.query = [seq.upper() for seq in self.query]
+
     self.lengths = sorted(set([len(peptide) for peptide in self.query]))
     self.proteome = proteome
     self.proteome_name = proteome.split('/')[-1].split('.')[0]
@@ -79,7 +95,7 @@ class Matcher(Preprocessor):
       self.k_specified = False
 
     # for exact matching, if no k is specified, use smallest length in query as k
-    if not max_mismatches and not k:
+    if not max_mismatches and not k and self.query:
       self.k = self.lengths[0]
 
     # for mismatching, if no k is specified, batch the peptides by length
@@ -99,15 +115,9 @@ class Matcher(Preprocessor):
 
     assert k >= 0, 'Invalid k value given.'
 
-    # if max_mismatches == -1:
-    #   assert best_match, 'Number of mismatches not specified.'
-
     if self.output_format not in VALID_OUTPUT_FORMATS:
       raise ValueError('Invalid output format, please choose dataframe, csv, xlsx, json, or html.')
 
-    if not all([seq.isupper() for seq in self.query]):
-      raise ValueError('A peptide in the query contains a lowercase letter.')
-
     super().__init__(self.proteome, self.preprocess_format, self.preprocessed_files_path, True)
 
   def batch_query(self):
@@ -533,11 +543,70 @@ class Matcher(Preprocessor):
       self.batched_peptides = {0: self.query}
 
     return all_matches
-      
+
+  def get_protein_metadata(self, protein_record):
+    '''Extracts protein metadata from a protein record in FASTA format.'''
+    protein_id = protein_record.id.split('|')[1] if '|' in protein_record.id else protein_record.id
+    
+    # use regex to get all other data from the UniProt FASTA header
+    try:
+      taxon_id = int(re.search('OX=(.*?) ', protein_record.description).group(1))
+    except AttributeError:
+      taxon_id = None
+
+    try:
+      species = re.search('OS=(.*) OX=', protein_record.description).group(1)
+    except AttributeError:
+      species = None
+
+    try:
+      gene = re.search('GN=(.*?) ', protein_record.description).group(1)
+    except AttributeError:
+      gene = None
+
+    try:
+      protein_name = re.search(' (.*) OS', protein_record.description).group(1)
+    except AttributeError:
+      protein_name = None
+
+    try:
+      pe_level = int(re.search('PE=(.*?) ', protein_record.description).group(1))
+    except AttributeError:
+      pe_level = 0
+
+    return (taxon_id, species, gene, protein_id, protein_name, pe_level)
+
+  def discontinuous_search(self):
+    """Find matches for discontinuous epitopes."""
+    all_matches = []
+    for discontinuous_epitope in self.discontinuous_epitopes:
+      for protein_record in parse_fasta(self.proteome):
+        try:
+          residue_matches = sum([x[0] == protein_record.seq[x[1] - 1] for x in discontinuous_epitope])
+          if residue_matches >= (len(discontinuous_epitope) - self.max_mismatches):
+            protein_metadata = self.get_protein_metadata(protein_record)
+            all_matches.append((', '.join([x[0] + str(x[1]) for x in discontinuous_epitope]), 
+                                ', '.join([protein_record.seq[x[1] - 1] + str(x[1]) for x in discontinuous_epitope]),
+                                protein_metadata[0],
+                                protein_metadata[1],
+                                protein_metadata[2],
+                                protein_metadata[3],
+                                protein_metadata[4],
+                                len(discontinuous_epitope) - residue_matches,
+                                [x[1] for x in discontinuous_epitope if x[0] != protein_record.seq[x[1] - 1]],
+                                discontinuous_epitope[0][1],
+                                discontinuous_epitope[-1][1],
+                                protein_metadata[5],
+                                np.nan))
+        except IndexError:
+          continue
+    
+    return all_matches
+
   def dataframe_matches(self, all_matches):
     '''Return Pandas dataframe of the results.'''
     df = pd.DataFrame(all_matches,
-                      columns=['Query Sequence', 'Matched Sequence', 'Proteome',
+                      columns=['Query Sequence', 'Matched Sequence', 'Taxon ID',
                                'Species', 'Gene', 'Protein ID', 'Protein Name',
                                'Mismatches', 'Mutated Positions', 'Index start', 
                                'Index end', 'Protein Existence Level', 'Gene Priority'])
@@ -566,7 +635,6 @@ class Matcher(Preprocessor):
 
     return df
 
-
   def output_matches(self, df):
     '''Write Pandas dataframe to format that is specified'''
     if self.output_format == 'csv':
@@ -583,14 +651,26 @@ class Matcher(Preprocessor):
     Overarching function that calls the appropriate search matching function
     based on the parameters.
     '''
-    if self.max_mismatches == -1:
-      df = self.dataframe_matches(self.best_match_search())
+    if self.query:
+      if self.max_mismatches == -1:
+        query_df = self.dataframe_matches(self.best_match_search())
 
-    elif self.max_mismatches > 0:
-      df = self.dataframe_matches(self.mismatching_search())
-    
+      elif self.max_mismatches > 0:
+        query_df = self.dataframe_matches(self.mismatching_search())
+      
+      else:
+        query_df = self.dataframe_matches(self.exact_match_search())
     else:
-      df = self.dataframe_matches(self.exact_match_search())
+      query_df = pd.DataFrame()
+
+    # search for discontinuous epitopes if they exist
+    if self.discontinuous_epitopes:
+      discontinuous_df = self.dataframe_matches(self.discontinuous_search())
+    else:
+      discontinuous_df = pd.DataFrame()
+
+    # combine the dataframes if both query and discontinuous epitopes exist
+    df = pd.concat([query_df, discontinuous_df], ignore_index=True)
 
     # return a dataframe instead of outputting file if specified
     if self.output_format == 'dataframe':
